@@ -34,6 +34,7 @@
 #include "session.h"
 #include "session_p.h"
 #include "statusadaptor.h"
+#include "agentconfigurationdialog.h"
 #include "private/standarddirs_p.h"
 
 #include "akonadiagentbase_debug.h"
@@ -50,6 +51,7 @@
 #include <QCommandLineParser>
 #include <QNetworkConfiguration>
 #include <QNetworkConfigurationManager>
+#include <QPointer>
 
 #include <signal.h>
 #include <stdlib.h>
@@ -481,6 +483,8 @@ void AgentBasePrivate::itemAdded(const Akonadi::Item &item, const Akonadi::Colle
 {
     if (mObserver) {
         mObserver->itemAdded(item, collection);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -488,6 +492,8 @@ void AgentBasePrivate::itemChanged(const Akonadi::Item &item, const QSet<QByteAr
 {
     if (mObserver) {
         mObserver->itemChanged(item, partIdentifiers);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -530,6 +536,8 @@ void AgentBasePrivate::itemRemoved(const Akonadi::Item &item)
 {
     if (mObserver) {
         mObserver->itemRemoved(item);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -601,6 +609,7 @@ void AgentBasePrivate::itemsLinked(const Akonadi::Item::List &items, const Akona
 void AgentBasePrivate::itemsUnlinked(const Akonadi::Item::List &items, const Akonadi::Collection &collection)
 {
     if (!mObserver) {
+        changeProcessed();
         return;
     }
 
@@ -614,10 +623,6 @@ void AgentBasePrivate::itemsUnlinked(const Akonadi::Item::List &items, const Ako
 
 void AgentBasePrivate::tagAdded(const Akonadi::Tag &tag)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->tagAdded(tag);
@@ -628,10 +633,6 @@ void AgentBasePrivate::tagAdded(const Akonadi::Tag &tag)
 
 void AgentBasePrivate::tagChanged(const Akonadi::Tag &tag)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->tagChanged(tag);
@@ -642,10 +643,6 @@ void AgentBasePrivate::tagChanged(const Akonadi::Tag &tag)
 
 void AgentBasePrivate::tagRemoved(const Akonadi::Tag &tag)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->tagRemoved(tag);
@@ -656,10 +653,6 @@ void AgentBasePrivate::tagRemoved(const Akonadi::Tag &tag)
 
 void AgentBasePrivate::itemsTagsChanged(const Akonadi::Item::List &items, const QSet<Akonadi::Tag> &addedTags, const QSet<Akonadi::Tag> &removedTags)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->itemsTagsChanged(items, addedTags, removedTags);
@@ -670,10 +663,6 @@ void AgentBasePrivate::itemsTagsChanged(const Akonadi::Item::List &items, const
 
 void AgentBasePrivate::relationAdded(const Akonadi::Relation &relation)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->relationAdded(relation);
@@ -684,10 +673,6 @@ void AgentBasePrivate::relationAdded(const Akonadi::Relation &relation)
 
 void AgentBasePrivate::relationRemoved(const Akonadi::Relation &relation)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->relationRemoved(relation);
@@ -700,10 +685,6 @@ void AgentBasePrivate::itemsRelationsChanged(const Akonadi::Item::List &items,
         const Akonadi::Relation::List &addedRelations,
         const Akonadi::Relation::List &removedRelations)
 {
-    if (!mObserver) {
-        return;
-    }
-
     AgentBase::ObserverV4 *observer4 = dynamic_cast<AgentBase::ObserverV4 *>(mObserver);
     if (observer4) {
         observer4->itemsRelationsChanged(items, addedRelations, removedRelations);
@@ -716,6 +697,8 @@ void AgentBasePrivate::collectionAdded(const Akonadi::Collection &collection, co
 {
     if (mObserver) {
         mObserver->collectionAdded(collection, parent);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -724,6 +707,8 @@ void AgentBasePrivate::collectionChanged(const Akonadi::Collection &collection)
     AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2 *>(mObserver);
     if (mObserver && observer2 == nullptr) {   // For ObserverV2 we use the variant with the part identifiers
         mObserver->collectionChanged(collection);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -732,6 +717,8 @@ void AgentBasePrivate::collectionChanged(const Akonadi::Collection &collection,
     AgentBase::ObserverV2 *observer2 = dynamic_cast<AgentBase::ObserverV2 *>(mObserver);
     if (observer2) {
         observer2->collectionChanged(collection, changedAttributes);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -754,6 +741,8 @@ void AgentBasePrivate::collectionRemoved(const Akonadi::Collection &collection)
 {
     if (mObserver) {
         mObserver->collectionRemoved(collection);
+    } else {
+        changeProcessed();
     }
 }
 
@@ -915,6 +904,7 @@ AgentBase::~AgentBase()
 
 void AgentBase::debugAgent(int argc, char **argv)
 {
+    Q_UNUSED(argc);
 #ifdef Q_OS_WIN
     if (qEnvironmentVariableIsSet("AKONADI_DEBUG_WAIT")) {
         if (QByteArray(argv[0]).endsWith(qgetenv("AKONADI_DEBUG_WAIT") + ".exe")) {
@@ -924,6 +914,8 @@ void AgentBase::debugAgent(int argc, char **argv)
             DebugBreak();
         }
     }
+#else
+    Q_UNUSED(argv);
 #endif
 }
 
@@ -1092,7 +1084,17 @@ void AgentBase::doSetOnline(bool online)
 void AgentBase::configure(WId windowId)
 {
     Q_UNUSED(windowId);
-    emit configurationDialogAccepted();
+
+    // Fallback if the agent implements the new plugin-based configuration,
+    // but someone calls the deprecated configure() method
+    auto instance = Akonadi::AgentManager::self()->instance(identifier());
+    QPointer<AgentConfigurationDialog> dialog = new AgentConfigurationDialog(instance, nullptr);
+    if (dialog->exec()) {
+        Q_EMIT configurationDialogAccepted();
+    } else {
+        Q_EMIT configurationDialogRejected();
+    }
+    delete dialog;
 }
 
 #ifdef Q_OS_WIN //krazy:exclude=cpp
