@@ -29,6 +29,7 @@
 #include "servermanager.h"
 #include "tagfetchscope.h"
 #include "persistentsearchattribute.h"
+#include "itemfetchscope.h"
 
 #include "private/protocol_p.h"
 #include "private/externalpartstorage_p.h"
@@ -380,17 +381,7 @@ Protocol::ItemFetchScope ProtocolHelper::itemFetchScopeToProtocol(const ItemFetc
     fs.setFetch(Protocol::ItemFetchScope::RemoteID, fetchScope.fetchRemoteIdentification());
     fs.setFetch(Protocol::ItemFetchScope::RemoteRevision, fetchScope.fetchRemoteIdentification());
     fs.setFetch(Protocol::ItemFetchScope::GID, fetchScope.fetchGid());
-    if (fetchScope.fetchTags()) {
-        fs.setFetch(Protocol::ItemFetchScope::Tags);
-        if (!fetchScope.tagFetchScope().fetchIdOnly()) {
-            if (fetchScope.tagFetchScope().attributes().isEmpty()) {
-                fs.setTagFetchScope({ "ALL" });
-            } else {
-                fs.setTagFetchScope(fetchScope.tagFetchScope().attributes());
-            }
-        }
-    }
-
+    fs.setFetch(Protocol::ItemFetchScope::Tags, fetchScope.fetchTags());
     fs.setFetch(Protocol::ItemFetchScope::VirtReferences, fetchScope.fetchVirtualReferences());
     fs.setFetch(Protocol::ItemFetchScope::MTime, fetchScope.fetchModificationTime());
     fs.setFetch(Protocol::ItemFetchScope::Relations, fetchScope.fetchRelations());
@@ -425,10 +416,10 @@ ItemFetchScope ProtocolHelper::parseItemFetchScope(const Protocol::ItemFetchScop
         ifs.setIgnoreRetrievalErrors(true);
     }
     switch (fetchScope.ancestorDepth()) {
-    case Protocol::Ancestor::ParentAncestor:
+        case Protocol::ItemFetchScope::ParentAncestor:
         ifs.setAncestorRetrieval(ItemFetchScope::Parent);
         break;
-    case Protocol::Ancestor::AllAncestors:
+        case Protocol::ItemFetchScope::AllAncestors:
         ifs.setAncestorRetrieval(ItemFetchScope::All);
         break;
     default:
@@ -446,14 +437,6 @@ ItemFetchScope ProtocolHelper::parseItemFetchScope(const Protocol::ItemFetchScop
     }
     if (fetchScope.fetch(Protocol::ItemFetchScope::Tags)) {
         ifs.setFetchTags(true);
-        const auto tfs = fetchScope.tagFetchScope();
-        if (tfs.isEmpty()) {
-            ifs.tagFetchScope().setFetchIdOnly(true);
-        } else if (QSet<QByteArray>({ "ALL" }) != tfs) {
-            for (const auto &attr : tfs) {
-                ifs.tagFetchScope().fetchAttribute(attr, true);
-            }
-        }
     }
     if (fetchScope.fetch(Protocol::ItemFetchScope::VirtReferences)) {
         ifs.setFetchVirtualReferences(true);
@@ -569,6 +552,8 @@ Protocol::TagFetchScope ProtocolHelper::tagFetchScopeToProtocol(const TagFetchSc
     Protocol::TagFetchScope tfs;
     tfs.setFetchIdOnly(fetchScope.fetchIdOnly());
     tfs.setAttributes(fetchScope.attributes());
+    tfs.setFetchRemoteID(fetchScope.fetchRemoteId());
+    tfs.setFetchAllAttributes(fetchScope.fetchAllAttributes());
     return tfs;
 }
 
@@ -576,6 +561,8 @@ TagFetchScope ProtocolHelper::parseTagFetchScope(const Protocol::TagFetchScope &
 {
     TagFetchScope tfs;
     tfs.setFetchIdOnly(fetchScope.fetchIdOnly());
+    tfs.setFetchRemoteId(fetchScope.fetchRemoteID());
+    tfs.setFetchAllAttributes(fetchScope.fetchAllAttributes());
     const auto attrs = fetchScope.attributes();
     for (const auto &attr : attrs) {
         tfs.fetchAttribute(attr, true);
@@ -607,13 +594,16 @@ static Item::Flags convertFlags(const QVector<QByteArray> &flags, ProtocolHelper
     return convertedFlags;
 }
 
-Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &data, ProtocolHelperValuePool *valuePool)
+Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &data,
+                                          const Akonadi::ItemFetchScope *fetchScope, ProtocolHelperValuePool *valuePool)
 {
     Item item;
     item.setId(data.id());
     item.setRevision(data.revision());
-    item.setRemoteId(data.remoteId());
-    item.setRemoteRevision(data.remoteRevision());
+    if (!fetchScope || fetchScope->fetchRemoteIdentification()) {
+        item.setRemoteId(data.remoteId());
+        item.setRemoteRevision(data.remoteRevision());
+    }
     item.setGid(data.gid());
     item.setStorageCollectionId(data.parentId());
 
@@ -629,7 +619,7 @@ Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &da
 
     item.setFlags(convertFlags(data.flags(), valuePool));
 
-    if (!data.tags().isEmpty()) {
+    if ((!fetchScope || fetchScope->fetchTags()) && !data.tags().isEmpty()) {
         Tag::List tags;
         tags.reserve(data.tags().size());
         Q_FOREACH (const Protocol::FetchTagsResponse &tag, data.tags()) {
@@ -638,7 +628,7 @@ Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &da
         item.setTags(tags);
     }
 
-    if (!data.relations().isEmpty()) {
+    if ((!fetchScope || fetchScope->fetchRelations()) && !data.relations().isEmpty()) {
         Relation::List relations;
         relations.reserve(data.relations().size());
         Q_FOREACH (const Protocol::FetchRelationsResponse &rel, data.relations()) {
@@ -647,7 +637,7 @@ Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &da
         item.d_ptr->mRelations = relations;
     }
 
-    if (!data.virtualReferences().isEmpty()) {
+    if ((!fetchScope || fetchScope->fetchVirtualReferences()) && !data.virtualReferences().isEmpty()) {
         Collection::List virtRefs;
         virtRefs.reserve(data.virtualReferences().size());
         Q_FOREACH (qint64 colId, data.virtualReferences()) {
@@ -674,6 +664,9 @@ Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &da
         const auto metaData = part.metaData();
         switch (ns) {
         case ProtocolHelper::PartPayload:
+            if (fetchScope && !fetchScope->fullPayload() && !fetchScope->payloadParts().contains(plainKey)) {
+                continue;
+            }
             ItemSerializer::deserialize(item, plainKey, part.data(), metaData.version(),
                                         static_cast<ItemSerializer::PayloadStorage>(metaData.storageType()));
             if (metaData.storageType() == Protocol::PartMetaData::Foreign) {
@@ -681,6 +674,9 @@ Item ProtocolHelper::parseItemFetchResult(const Protocol::FetchItemsResponse &da
             }
             break;
         case ProtocolHelper::PartAttribute: {
+            if (fetchScope && !fetchScope->allAttributes() && !fetchScope->attributes().contains(plainKey)) {
+                continue;
+            }
             Attribute *attr = AttributeFactory::createAttribute(plainKey);
             Q_ASSERT(attr);
             if (metaData.storageType() == Protocol::PartMetaData::External) {
