@@ -33,8 +33,9 @@ namespace Server
 */
 class Login : public Handler
 {
-    Q_OBJECT
 public:
+    ~Login() override = default;
+
     bool parseStream() override;
 };
 
