@@ -37,9 +37,10 @@ namespace Server
 */
 class Modify : public Handler
 {
-    Q_OBJECT
 public:
-    bool parseStream() Q_DECL_OVERRIDE;
+    ~Modify() override = default;
+
+    bool parseStream() override;
 };
 
 } // namespace Server
