@@ -32,24 +32,17 @@ namespace Server
 
   Handler for the item deletion command.
 
-  <h4>Syntax</h4>
-  One of the following three:
-  @verbatim
-  <tag> REMOVE <uid-set>
-  <tag> UID REMOVE <uid-set>
-  <tag> RID REMOVE <remote-identifier>
-  @endverbatim
-
   <h4>Semantics</h4>
   Removes the selected items. Item selection can happen within the usual three scopes:
   - based on a uid set relative to the currently selected collection
   - based on a global uid set (UID)
   - based on a remote identifier within the currently selected collection (RID)
 */
-class  Remove : public Handler
+class Remove : public Handler
 {
-    Q_OBJECT
 public:
+    ~Remove() override = default;
+
     bool parseStream() override;
 };
 
