@@ -31,6 +31,7 @@
 #include "aggregatedfetchscope.h"
 #include "selectquerybuilder.h"
 #include "handler/fetchhelper.h"
+#include "connection.h"
 
 #include "akonadiserver_debug.h"
 
@@ -242,9 +243,9 @@ void NotificationCollector::clear()
     mNotifications.clear();
 }
 
-void NotificationCollector::setSessionId(const QByteArray &sessionId)
+void NotificationCollector::setConnection(Connection *connection)
 {
-    mSessionId = sessionId;
+    mConnection = connection;
 }
 
 void NotificationCollector::itemNotification(Protocol::ItemChangeNotification::Operation op,
@@ -282,7 +283,9 @@ void NotificationCollector::itemNotification(Protocol::ItemChangeNotification::O
     }
 
     auto msg = Protocol::ItemChangeNotificationPtr::create();
-    msg->setSessionId(mSessionId);
+    if (mConnection) {
+        msg->setSessionId(mConnection->sessionId());
+    }
     msg->setOperation(op);
 
     msg->setItemParts(parts);
@@ -383,7 +386,9 @@ void NotificationCollector::collectionNotification(Protocol::CollectionChangeNot
 {
     auto msg = Protocol::CollectionChangeNotificationPtr::create();
     msg->setOperation(op);
-    msg->setSessionId(mSessionId);
+    if (mConnection) {
+        msg->setSessionId(mConnection->sessionId());
+    }
     msg->setParentCollection(source);
     msg->setParentDestCollection(destination);
     msg->setDestinationResource(destResource);
@@ -457,14 +462,17 @@ void NotificationCollector::tagNotification(Protocol::TagChangeNotification::Ope
 {
     auto msg = Protocol::TagChangeNotificationPtr::create();
     msg->setOperation(op);
-    msg->setSessionId(mSessionId);
+    if (mConnection) {
+        msg->setSessionId(mConnection->sessionId());
+    }
     msg->setResource(resource);
-    auto msgTag = HandlerHelper::fetchTagsResponse(tag, false);
+    Protocol::FetchTagsResponse msgTag;
+    msgTag.setId(tag.id());
     msgTag.setRemoteId(remoteId.toUtf8());
     if (auto mgr = AkonadiServer::instance()->notificationManager()) {
         auto fetchScope = mgr->tagFetchScope();
         if (!fetchScope->fetchIdOnly() && msgTag.gid().isEmpty()) {
-            msgTag = HandlerHelper::fetchTagsResponse(Tag::retrieveById(msgTag.id()), false);
+            msgTag = HandlerHelper::fetchTagsResponse(Tag::retrieveById(msgTag.id()), fetchScope->toFetchScope(), mConnection);
         }
 
         const auto requestedAttrs = fetchScope->attributes();
@@ -499,7 +507,9 @@ void NotificationCollector::relationNotification(Protocol::RelationChangeNotific
 {
     auto msg = Protocol::RelationChangeNotificationPtr::create();
     msg->setOperation(op);
-    msg->setSessionId(mSessionId);
+    if (mConnection) {
+        msg->setSessionId(mConnection->sessionId());
+    }
     msg->setRelation(HandlerHelper::fetchRelationsResponse(relation));
 
     dispatchNotification(msg);
@@ -535,15 +545,19 @@ void NotificationCollector::completeNotification(const Protocol::ChangeNotificat
             // we emit a notification without it and leave it up to the Monitor
             // to retrieve the Item on demand - we should have a RID stored in
             // Akonadi by then.
-            if (Connection::self() && (allHaveRID || msg->operation() != Protocol::ItemChangeNotification::Add)) {
+            if (mConnection && (allHaveRID || msg->operation() != Protocol::ItemChangeNotification::Add)) {
 
                 // Prevent transactions inside FetchHelper to recursively call our slot
                 QScopedValueRollback<bool> ignoreTransactions(mIgnoreTransactions);
                 mIgnoreTransactions = true;
                 CommandContext context;
-                auto scope = fetchScope->toFetchScope();
-                scope.setFetch(Protocol::ItemFetchScope::CacheOnly);
-                FetchHelper helper(Connection::self(), &context, Scope(ids), scope);
+                auto itemFetchScope = fetchScope->toFetchScope();
+                auto tagFetchScope = mgr->tagFetchScope()->toFetchScope();
+                itemFetchScope.setFetch(Protocol::ItemFetchScope::CacheOnly);
+                FetchHelper helper(mConnection, &context, Scope(ids), itemFetchScope, tagFetchScope);
+                // The Item was just changed, which means the atime was
+                // updated, no need to do it again a couple milliseconds later.
+                helper.disableATimeUpdates();
                 QVector<Protocol::FetchItemsResponse> fetchedItems;
                 auto callback = [&fetchedItems](Protocol::FetchItemsResponse &&cmd) {
                     fetchedItems.push_back(std::move(cmd));
