@@ -39,7 +39,7 @@ namespace Internal
 class AgentTypeWidgetDelegate : public QAbstractItemDelegate
 {
 public:
-    AgentTypeWidgetDelegate(QObject *parent = nullptr);
+    explicit AgentTypeWidgetDelegate(QObject *parent = nullptr);
 
     void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
     QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override;
